#pragma once
#include <exception>
#include <opencv2/core/mat.hpp>
#include <opencv2/objdetect/face.hpp>
#include <opencv2/core.hpp>
#include <vector>
#include <memory>
#include <filesystem>

class FaceRecognizer
{
public:

	class LoadException : public std::exception
	{
	private:
		std::string message;
	public:
		LoadException(const std::string& msg): std::exception(), message(msg) {}
		virtual const char* what() const throw() override
		{
			return message.c_str();
		}
	};

private:
	std::vector<cv::Mat> referanceFeatures;
	std::shared_ptr<cv::FaceRecognizerSF> recognizer;
	std::shared_ptr<cv::FaceDetectorYN> detector;

	double threshold = 0.363;

public:
	FaceRecognizer(std::filesystem::path recognizerPath = "", const std::filesystem::path& detectorPath = "", const std::vector<cv::Mat>& referances = std::vector<cv::Mat>());
	cv::Mat detectFaces(const cv::Mat& input);
	std::pair<int, double> isMatch(const cv::Mat& input, bool alone = false);
	bool addReferances(const std::vector<cv::Mat>& referances);
	void setThreshold(double threashold);
	double getThreshold();
	void clearReferances();
};
